@@ -594,6 +594,11 @@ pub trait KeyHandle {
     /// 5 #217813388320."
     fn id(&self) -> String;
 
+    /// Returns the key's handle.
+    fn key_handle(&self) -> openpgp::KeyHandle {
+        self.fingerprint().into()
+    }
+
     /// Returns the key's fingerprint.
     fn fingerprint(&self) -> Fingerprint;
 
@@ -665,7 +670,7 @@ pub trait KeyHandle {
     /// Decrypts a PKESK.
     ///
     async fn decrypt_pkesk(&mut self, pkesk: &PKESK)
-        -> Option<(SymmetricAlgorithm, SessionKey)>
+        -> Option<(Option<SymmetricAlgorithm>, SessionKey)>
     {
         // We want to use `PKESK::decrypt`.  For that, we need
         // something that implements the `Decryptor` interface.  We
