@@ -4,7 +4,6 @@ use sequoia_openpgp as openpgp;
 
 use openpgp::Cert;
 use openpgp::Fingerprint;
-use openpgp::KeyID;
 use openpgp::Result;
 use openpgp::crypto::Password;
 use openpgp::crypto::mpi;
@@ -90,7 +89,7 @@ impl VerificationHelper for &mut VerifyHelper {
            .filter(|cert| {
                cert.keys().any(|k| {
                    ids.iter().any(|id| {
-                       id.aliases(openpgp::KeyHandle::from(k.fingerprint()))
+                       id.aliases(openpgp::KeyHandle::from(k.key().fingerprint()))
                    })
                })
            })
@@ -105,7 +104,7 @@ impl VerificationHelper for &mut VerifyHelper {
                 MessageLayer::SignatureGroup { ref results } => {
                     for r in results {
                         if let Ok(GoodChecksum { ka, .. }) = r {
-                            self.signers.push(ka.fingerprint());
+                            self.signers.push(ka.key().fingerprint());
                         } else {
                             log::info!("Bad signature: {:?}", r);
                         }
@@ -136,9 +135,9 @@ pub fn sign_verify(signers: Vec<TestSigner>,
 
     let mut signers = signers.into_iter();
     let mut signer = Signer::with_template(
-        message, signers.next().expect("a signer"), builder);
+        message, signers.next().expect("a signer"), builder)?;
     for s in signers {
-        signer = signer.add_signer(s);
+        signer = signer.add_signer(s)?;
     }
     let signer = signer.build().context("Failed to create signer")?;
     let mut writer = LiteralWriter::new(signer).build()
@@ -185,7 +184,8 @@ pub fn sign_verify(signers: Vec<TestSigner>,
 /// This function tries to decrypt all of the keys with the password.
 pub async fn try_decrypt<B>(backend: &mut B,
                             ciphertext: &[u8], plaintext: Option<&[u8]>,
-                            keyid: &KeyID, password: Option<Password>)
+                            recipient: openpgp::KeyHandle,
+                            password: Option<Password>)
     -> Result<()>
 where B: Backend
 {
@@ -195,15 +195,23 @@ where B: Backend
     let mut ppr = PacketParser::from_bytes(ciphertext)?;
     while let PacketParserResult::Some(mut pp) = ppr {
         match pp.packet {
-            Packet::PKESK(ref pkesk) if pkesk.recipient() == keyid => {
+            Packet::PKESK(ref pkesk)
+                if pkesk.recipient().as_ref() == Some(&recipient) =>
+            {
                 // Manual search.
-                log::debug!("Considering PKESK for {}", pkesk.recipient());
+                log::debug!("Considering PKESK for {}",
+                            pkesk.recipient().unwrap());
 
                 'done: for device in backend.list().await {
                     for mut key in device.list().await {
                         log::debug!("Considering key {}", key.fingerprint());
 
-                        if pkesk.recipient() != &key.keyid() {
+                        let key_handle =
+                            openpgp::KeyHandle::from(key.fingerprint());
+                        if pkesk.recipient()
+                            .map(|r| ! r.aliases(&key_handle))
+                            .unwrap_or(false)
+                        {
                             log::debug!("Wrong key for PKESK {}", key.fingerprint());
                             continue;
                         }
@@ -233,9 +241,9 @@ where B: Backend
                     }
                 }
             }
-            Packet::PKESK(ref pkesk) if pkesk.recipient() == keyid => {
-                // Manual search.
-                log::debug!("Ignoring PKESK for {}", pkesk.recipient());
+            Packet::PKESK(ref pkesk) => {
+                // Catch-all for packets that failed the manual search.
+                log::debug!("Ignoring PKESK for {:?}", pkesk.recipient());
             }
             Packet::SEIP(_) => {
                 if let Some(session_key) = session_key.take() {
@@ -336,12 +344,12 @@ macro_rules! generate_tests {
         /// This is useful when `$key_sets` is `Some` and the backend
         /// doesn't have space for the primary key.
         fn strip_primary_secret(cert: Cert) -> Cert {
-            if cert.primary_key().has_secret() {
+            if cert.primary_key().key().has_secret() {
                 let mut pk = cert.primary_key().key().clone();
                 pk.steal_secret();
 
-                let cert = cert.insert_packets(pk).expect("can merge");
-                assert!(! cert.primary_key().has_secret());
+                let cert = cert.insert_packets(pk).expect("can merge").0;
+                assert!(! cert.primary_key().key().has_secret());
                 cert
             } else {
                 cert
@@ -384,8 +392,8 @@ macro_rules! generate_tests {
             imported_keys.sort();
 
             let mut expected_keys = Vec::new();
-            for key in cert.keys().secret() {
-                expected_keys.push(key.fingerprint());
+            for ka in cert.keys().secret() {
+                expected_keys.push(ka.key().fingerprint());
             }
             expected_keys.sort();
 
@@ -431,8 +439,8 @@ macro_rules! generate_tests {
                 .expect("valid cert");
 
             let mut expected_keys = Vec::new();
-            for key in cert.keys().secret() {
-                expected_keys.push(key.fingerprint());
+            for ka in cert.keys().secret() {
+                expected_keys.push(ka.key().fingerprint());
             }
             expected_keys.sort();
 
@@ -530,14 +538,13 @@ macro_rules! generate_tests {
             $import_cert(&mut backend, &cert).await;
 
             let msg: &[u8] = $crate::test_framework::data::message("no-password.asc");
-            let keyid: sequoia_openpgp::KeyID
-                = "AA5BFAC1E4A121A0".parse().expect("valid");
+            let keyid = KeyHandle::KeyID("AA5BFAC1E4A121A0".parse()?);
 
             let default_password: Option<&str> = $default_password;
             let default_password: Option<Password>
                 = default_password.map(|p| Password::from(p));
             if let Err(err) = $crate::test_framework::try_decrypt(
-                &mut backend, msg, Some(b"foo\n"), &keyid, default_password).await
+                &mut backend, msg, Some(b"foo\n"), keyid, default_password).await
             {
                 eprintln!("Decrypting: {}", err);
                 Err(err)
@@ -583,12 +590,11 @@ macro_rules! generate_tests {
             $import_cert(&mut backend, &cert).await;
 
             let msg: &[u8] = $crate::test_framework::data::message("bob.asc");
-            let keyid: sequoia_openpgp::KeyID
-                = "0A4BA2249168D779".parse().expect("valid");
+            let keyid = KeyHandle::KeyID("0A4BA2249168D779".parse()?);
 
             assert!(
                 $crate::test_framework::try_decrypt(
-                    &mut backend, msg, Some(b"hi bob\n"), &keyid, None)
+                    &mut backend, msg, Some(b"hi bob\n"), keyid, None)
                     .await.is_err());
 
             Ok(())
@@ -624,8 +630,7 @@ macro_rules! generate_tests {
             let mut backend = $init_backend().await;
 
             let msg: &[u8] = $crate::test_framework::data::message("bob.asc");
-            let keyid: sequoia_openpgp::KeyID
-                = "0A4BA2249168D779".parse().expect("valid");
+            let keyid = KeyHandle::KeyID("0A4BA2249168D779".parse()?);
 
             // Import Bob's certificate.  The keys are password
             // protected.  Make sure we can use them with the right
@@ -640,13 +645,13 @@ macro_rules! generate_tests {
             assert!(
                 $crate::test_framework::try_decrypt(
                     &mut backend, msg, Some(b"hi bob\n"),
-                    &keyid, Some(Password::from("sup3r s3cur3")))
+                    keyid.clone(), Some(Password::from("sup3r s3cur3")))
                     .await.is_err());
 
             log::info!("Decrypting with right password:");
             $crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, Some(Password::from("bob"))).await.unwrap();
+                keyid.clone(), Some(Password::from("bob"))).await.unwrap();
 
             // Import a new version of Bob's certificate in which the
             // keys are still encrypted, but with a different
@@ -661,11 +666,11 @@ macro_rules! generate_tests {
             log::info!("Decrypting with wrong password:");
             assert!($crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, Some(Password::from("sup3r s3cur3"))).await.is_err());
+                keyid.clone(), Some(Password::from("sup3r s3cur3"))).await.is_err());
             log::info!("Decrypting with right password:");
             $crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, Some(Password::from("smithy"))).await.unwrap();
+                keyid.clone(), Some(Password::from("smithy"))).await.unwrap();
 
             // Import a third version of Bob's certificate in which
             // the keys are not encrypt.
@@ -685,7 +690,7 @@ macro_rules! generate_tests {
             log::info!("Decrypting with right password:");
             $crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, None).await.unwrap();
+                keyid.clone(), None).await.unwrap();
 
             Ok(())
         }
@@ -816,7 +821,7 @@ macro_rules! generate_tests {
                 // Strip the primary.
                 cert_a = strip_primary_secret(cert_a);
             }
-            let keys_a = cert_a.keys().secret().map(|k| k.fingerprint())
+            let keys_a = cert_a.keys().secret().map(|k| k.key().fingerprint())
                 .collect::<Vec<Fingerprint>>();
 
             let keyring_b: &[u8] = $crate::test_framework::data::key(
@@ -827,7 +832,7 @@ macro_rules! generate_tests {
                 // Strip the primary.
                 cert_b = strip_primary_secret(cert_b);
             }
-            let keys_b = cert_b.keys().secret().map(|k| k.fingerprint())
+            let keys_b = cert_b.keys().secret().map(|k| k.key().fingerprint())
                 .collect::<Vec<Fingerprint>>();
 
             // We expect the same certificate, but with different
@@ -928,7 +933,7 @@ macro_rules! generate_tests {
 
             let (keys, public_keys): (Vec<(Fingerprint, bool)> , _) = cert
                 .keys()
-                .map(|k| (k.fingerprint(), k.has_secret()))
+                .map(|k| (k.key().fingerprint(), k.key().has_secret()))
                 .partition(|(_fpr, has_secret)| *has_secret);
 
             let keys = keys.into_iter().map(|(fpr, _)| fpr)
@@ -1003,7 +1008,7 @@ macro_rules! generate_tests {
                 let mut found = false;
                 'find: for mut d in backend.list().await {
                     for mut k in d.list().await {
-                        if k.fingerprint() == key.fingerprint() {
+                        if k.fingerprint() == key.key().fingerprint() {
                             let exported_key
                                 = k.export().await.expect("can export");
                             assert_eq!(&exported_key, key.key());
@@ -1014,7 +1019,7 @@ macro_rules! generate_tests {
                     }
                 }
 
-                assert!(found, "{} was not imported", key.fingerprint());
+                assert!(found, "{} was not imported", key.key().fingerprint());
             }
 
             // Make sure we tested something.
@@ -1053,7 +1058,7 @@ macro_rules! generate_tests {
             let fpr_str = "86353DE523CB334DC0B1F0F00A4BA2249168D779";
             let fpr: sequoia_openpgp::Fingerprint
                 = fpr_str.parse().expect("valid");
-            let keyid = sequoia_openpgp::KeyID::from(fpr);
+            let keyid = KeyHandle::KeyID(fpr.into());
 
             // Import Bob's certificate.  They keys are password
             // protected.  Make sure we can use them with the right
@@ -1068,13 +1073,13 @@ macro_rules! generate_tests {
             assert!(
                 $crate::test_framework::try_decrypt(
                     &mut backend, msg, Some(b"hi bob\n"),
-                    &keyid, Some(Password::from("sup3r s3cur3")))
+                    keyid.clone(), Some(Password::from("sup3r s3cur3")))
                     .await.is_err());
 
             log::info!("Decrypting with right password:");
             $crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, Some(Password::from("bob"))).await.unwrap();
+                keyid.clone(), Some(Password::from("bob"))).await.unwrap();
 
             // Change the password.
             let mut key = backend.find_key(fpr_str)
@@ -1099,15 +1104,15 @@ macro_rules! generate_tests {
             log::info!("Decrypting with wrong password:");
             assert!($crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, Some(Password::from("sup3r s3cur3"))).await.is_err());
+                keyid.clone(), Some(Password::from("sup3r s3cur3"))).await.is_err());
             log::info!("Decrypting with old password:");
             assert!($crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, Some(Password::from("smithy"))).await.is_err());
+                keyid.clone(), Some(Password::from("smithy"))).await.is_err());
             log::info!("Decrypting with new password:");
             $crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, Some(Password::from("new password"))).await.unwrap();
+                keyid.clone(), Some(Password::from("new password"))).await.unwrap();
 
             Ok(())
         }
@@ -1144,7 +1149,7 @@ macro_rules! generate_tests {
             let fpr_str = "86353DE523CB334DC0B1F0F00A4BA2249168D779";
             let fpr: sequoia_openpgp::Fingerprint
                 = fpr_str.parse().expect("valid");
-            let keyid = sequoia_openpgp::KeyID::from(fpr);
+            let keyid = KeyHandle::KeyID(fpr.into());
 
             // Import Bob's certificate.  They keys are password
             // protected.  Make sure we can use them with the right
@@ -1159,13 +1164,13 @@ macro_rules! generate_tests {
             assert!(
                 $crate::test_framework::try_decrypt(
                     &mut backend, msg, Some(b"hi bob\n"),
-                    &keyid, Some(Password::from("sup3r s3cur3")))
+                    keyid.clone(), Some(Password::from("sup3r s3cur3")))
                     .await.is_err());
 
             log::info!("Decrypting with right password:");
             $crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, Some(Password::from("bob"))).await.unwrap();
+                keyid.clone(), Some(Password::from("bob"))).await.unwrap();
 
             // Delete the primary key.
             log::info!("Deleting primary key:");
@@ -1189,7 +1194,7 @@ macro_rules! generate_tests {
             log::info!("Decrypting with right password:");
             $crate::test_framework::try_decrypt(
                 &mut backend, msg, Some(b"hi bob\n"),
-                &keyid, Some(Password::from("bob"))).await.unwrap();
+                keyid.clone(), Some(Password::from("bob"))).await.unwrap();
 
             log::info!("Deleting decryption key:");
             let mut key = backend.find_key(&fpr_str)
@@ -1202,7 +1207,7 @@ macro_rules! generate_tests {
             assert!(
                 $crate::test_framework::try_decrypt(
                     &mut backend, msg, Some(b"hi bob\n"),
-                    &keyid, Some(Password::from("bob")))
+                    keyid.clone(), Some(Password::from("bob")))
                     .await.is_err());
 
             // Make sure we can't find the decryption key any more.
